$regfile = "attiny26.dat"
$crystal = 1000000                                          ' 1 MHz

$hwstack = 32                                               ' default use 32 for the hardware stack
$swstack = 10                                               ' default use 10 for the SW stack
$framesize = 40

Declare Sub Updatedisplay()
Declare Sub Setsegments(b As Byte)

Config Adc = Single , Prescaler = Auto , Reference = Avcc   'Initialize ADC

'7-segment pins
Segment_a Alias Porta.2
Segment_b Alias Porta.0
Segment_c Alias Portb.3
Segment_d Alias Portb.1
Segment_e Alias Portb.2
Segment_f Alias Porta.1
Segment_g Alias Porta.3

'7-segment cathodes
Ones Alias Portb.0
Tens Alias Porta.5

'Encoder pushbutton
Switch Alias Pinb.6

Const Pot = 7                                               'PortA
Const Led = 4                                               'PortA
Const Buzzer = 5                                            'PortB

'Set port A pins  (0 = Input, 1 = Output)
'All output except 6 & 7 for the encoder inputs
Ddra = &B00111111
'Set internal pull-up resistors (0 = Off, 1 = On)
'6 & 7 are the only inputs
'Turn off all output ports (0 = Off, 1 = On)
Porta = &B11000000

'Set port B pins  (0 = Input, 1 = Output)
'All output except 4 & 6 for Pot & Pushbutton
Ddrb = &B00101111
'Set internal pull-up resistors (0 = Off, 1 = On)
'6 is the only inputs
'Turn off all output ports (0 = Off, 1 = On)
Portb = &B01000000

'Set up timer to track time
Config Timer0 = Timer , Prescale = 1024
Stop Timer0
'Preload 8 bit Timer Constant For 1/4 Second Duration At 1 Mhz for 1024 prescale
'Value should be around 12 (if my math is right) but my internal oscillator appears to be a little off.
'Either calibrate the AVR or tune the Timer1pre for accurate time keeping
Const Timer1pre = 75
Timer0 = Timer1pre
'Handle time overflow (occurs every 1/4 second)
On Timer0 Pulse

Enable Interrupts
Enable Timer0

Dim 7segment As Byte
Dim C As Byte
Dim Digit01 As Byte
Dim Digit10 As Byte

Dim Quarters As Byte
Quarters = 0

Dim Buzz_on As Bit
Buzz_on = 0
Dim Led_on As Bit
Led_on = 0

Dim Buzzer_level As Byte
Dim Counter_max As Byte
Counter_max = 30
Dim Counter_max10 As Byte
Counter_max10 = Counter_max / 10

Dim W As Word

Dim Switch_down As Byte
Switch_down = 0

Dim Counting As Bit
Counting = 0

'My encoder seems to generate 4 pulses for each detent so I added counters to only fire 1 pulse per detent
Dim Counting_down As Byte
Dim Counting_up As Byte

Dim E As Byte

'Used to skip first pulse of encoder.  My encoder fires a pulse when powered on.
Dim Program_ready As Bit

'Initialize maximum time
Gosub Set_max

Do
    'Update LED 7-segment display
    Updatedisplay

    'Check for LED
    If Led_on = 1 Then
       Pulseout Porta , Led , 5
    End If

    'Get buzzer level
    Gosub Poll_pot
    'Check for beep
    If Buzzer_level <> 0 And Buzz_on = 1 Then
      Pulseout Portb , Buzzer , Buzzer_level
    End If

   'Check encoder
    E = Encoder(pina.7 , Pina.6 , Rotate_cw , Rotate_ccw , 0)

    'Check start/stop switch
    If Switch = 0 Then
      'Switch pressed down
      Switch_down = 1
    Elseif Switch = 1 And Switch_down = 1 Then
      'Switch released
      Switch_down = 0
      If Counting = 0 Then
         'Start counter
         Counting = 1
         C = 0
         Digit01 = 0
         Digit10 = 0
         Timer0 = Timer1pre
         Start Timer0
      Else
         'Stop timer
         Counting = 0
         Stop Timer0
         'Display maximum time
         Gosub Set_max
      End If
    End If
Loop


End

'Turn on/off LED segments
Sub Setsegments(b As Byte)
   Segment_a = B.0
   Segment_b = B.1
   Segment_c = B.2
   Segment_d = B.3
   Segment_e = B.4
   Segment_f = B.5
   Segment_g = B.6
End Sub


Sub Updatedisplay()
   'Set 1 cathode
   Ones = 0
   Tens = 1

   'Ones digit
   'Get byte pattern
   7segment = Lookup(digit01 , Ledsegment)
   Setsegments 7segment

   Waitus 64
   7segment = &B00000000
   Setsegments 7segment

   'Set 10's cathode
   Ones = 1
   Tens = 0

   '10's Digit
   'Get byte pattern
   7segment = Lookup(digit10 , Ledsegment)
   Setsegments 7segment

   Waitus 64
   7segment = &B00000000
   Setsegments 7segment
End Sub


Rotate_cw:
   'Rotare encoder turned clockwise
   If Program_ready = 1 Then
      If Counter_max < 90 Then
         Incr Counting_up
         '4 pulses = 1 detent
         If Counting_up = 4 Then
            Counting_up = 0
            Counter_max = Counter_max + 10
            Gosub Set_max
         End If
      End If
   End If
Return

Rotate_ccw:
   'Rotary encoder turned counter-clockwise
   If Program_ready = 1 Then
      If Counter_max > 0 Then
         Incr Counting_down
         '4 pulses = 1 detent
         If Counting_down = 4 Then
            Counting_down = 0
            Counter_max = Counter_max - 10
            Gosub Set_max
         End If
      End If
   End If
Return


Pulse:
   'Increment 1/4 second counter
   Incr Quarters
   'Check for full second
   If Quarters = 4 Then
       Stop Timer0
      'Increment counter
       Incr C

      'Determine Ones digit
       Digit01 = C Mod 10

       'Determine Tens Digit
       Digit10 = C / 10

      'Reset 1/4 second counter
      Quarters = 0

      'Reset counter when maximum reached
      If Digit10 >= Counter_max10 And Digit01 > 0 Then
         C = 0
         Digit01 = 0
         Digit10 = 0
         'Turn LED and Buzzer off
         Buzz_on = 0
         Led_on = 0
      Elseif Digit10 >= Counter_max10 Then
         'Turn on LED and buzzer for last second
         Led_on = 1
         'Buzzer flag on
         Buzz_on = 1
      End If

      Timer0 = Timer1pre
      Start Timer0
   End If
Return


'Set the maximum time
Set_max:
   Stop Timer0
   Counting = 0
   Quarters = 0
   Led_on = 0
   Buzz_on = 0
   'Determine Ones digit
   Digit01 = Counter_max Mod 10

   'Determine Tens Digit
   Digit10 = Counter_max / 10

   'Set the maximum 10 digit of counter
   Counter_max10 = Digit10
   Program_ready = 1
Return


'Check  pot
Poll_pot:
   W = Getadc(pot)
   'Lower 10 bit ADC to 6 bit resolution
   Shift W , Right , 4
   Buzzer_level = W
Return


'Lookup table for LED digits
Ledsegment:
Data &B00111111 , &B00000110 , &B01011011 , &B01001111 , &B01100110 , &B01101101 , &B01111101 , &B00000111 , &B01111111 , &B01101111